const nsIWindowWatcher = Components.interfaces.nsIWindowWatcher;
const nsIRDFDataSource = Components.interfaces.nsIRDFDataSource;
const nsITimer         = Components.interfaces.nsITimer;

/* This timer block of code is used because the JavaScript here will run
 * before the XUL overlay is complete.  Therefore, the getElementById
 * will fail, causing the whole thing to come tumbling down.  Instead,
 * we wait until loading is complete, and then bind the event listener.
 */
var gRDFOverlayStartupHandler = {
    observe : function(aSubject, aTopic, aData) {
        if (aTopic == "timer-callback") {
            var popup = document.getElementById("popupContext");
            popup.addEventListener("popupshowing", openObjectPopup, false);
        }
    }
};

var gRDFOverlayStartupTimer = Components
    .classes["@mozilla.org/timer;1"]
    .createInstance(Components.interfaces.nsITimer);
gRDFOverlayStartupTimer.init(gRDFOverlayStartupHandler, 500, nsITimer.TYPE_ONE_SHOT);

/* This popup is used to determine if the currently-selected item is
 * an RDF datasource, and set the disabled state of this menuitem
 * accordingly.
 */
function openObjectPopup(event) {
    var selectedItem = getSelectedObject();
    var menuItem = document.getElementById("menu_inspectDataSource");
    var disableState = true;

    // Check if the currently-selected value exists, and is a datasource
    if (selectedItem.__JSValue__) {
        if (selectedItem.__JSValue__ instanceof nsIRDFDataSource) {
            disableState = false;
        }
    }

    // Set the disabled attribute, whatever it may have been determined to be
    menuItem.setAttribute("disabled", disableState);
    return true;
}

/* Find the menu popup, and browse through the DOM until we find the
 * appropriate item.
 */
function getSelectedObject() {
    var tree = document.popupNode.parentNode;
    var selectedIndex = tree.view.selection.currentIndex;
    var selectedItem = tree.view.getItemAtIndex(selectedIndex);
    return selectedItem;
}

/* Called when the menu item is clicked, this will open the current datasource
 * in either an already open instance of RDF Inspector, or in a new window.
 */
function inspectDataSource(event) {
    var windowURI = 'chrome://rdfinspector/content/rdfinspector.xul';
    var windowName = 'rdfinspector';
    var windowOptions = 'chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar';

    var selectedItem = getSelectedObject();
    if (selectedItem.__JSValue__) {
        if (selectedItem.__JSValue__ instanceof nsIRDFDataSource) {
            // Fetch a window watcher
            var ww = Components
                .classes["@mozilla.org/embedcomp/window-watcher;1"]
                .getService(nsIWindowWatcher);

            // Get an enumerator and iterate through all the open windows
            var windowItr = ww.getWindowEnumerator();
            var rdfWindow;
            while (windowItr.hasMoreElements()) {
                var win = windowItr.getNext();
                if (win.loadDataSource) {
                    // We have found the RDF Inspector window; set it aside
                    rdfWindow = win;
                    break;
                }
            }

            // Load the selected datasource in the currently-open RDF Inspector
            if (rdfWindow)
                rdfWindow.loadDataSource(selectedItem.__JSValue__);
            
            // Open a new RDF Inpspector window, with the supplied datasource
            else
                window.openDialog(windowURI, windowName, windowOptions, selectedItem.__JSValue__);
        }
    }
}
