var RDF = Components
    .classes['@mozilla.org/rdf/rdf-service;1']
    .getService()
    .QueryInterface(Components.interfaces.nsIRDFService);

var editButton;
var removeButton;
var namespaceTree;
var namespaces;
var nsTreeView;
var strings;

function onLoad() {
    editButton = document.getElementById("editButton");
    removeButton = document.getElementById("removeButton");
    namespaceTree = document.getElementById("namespaceTree");

    strings = document.getElementById('editNamespaceStrings');

    nsTreeView = new nsNamespaceTreeView(namespaces);
    nsTreeView.update();
    namespaceTree.view = nsTreeView;
}

function onAccept() {
    window.opener.updateNamespaceList();
}

function selectNamespace(aEvent) {
    var selection = namespaceTree.view.selection;
    if (selection.count > 0) removeButton.setAttribute('disabled', 'false');
    else removeButton.setAttribute('disabled', 'true');

    if (selection.count == 1) editButton.setAttribute('disabled', 'false');
    else editButton.setAttribute('disabled', 'true');
}

function importNamespaces(aEvent) {
    if (!confirm(strings.getString('importConfirmation')))
        return;

    var nsDS = RDF.GetDataSourceBlocking('http://nachbaur.com/~mike/rdf/commonNamespaces.rdf');
    if (!nsDS instanceof Components.interfaces.nsIRDFDataSource) {
        alert(strings.getString('importFailed'));
        return;
    }

    var rdfType = RDF.GetResource('http://www.w3.org/1999/02/22-rdf-syntax-ns#type');
    var rdfNamespace = RDF.GetResource('http://rdfinspector.mozdev.org/rdf/2005/Namespace');
    var rdfPrefix = RDF.GetResource('http://rdfinspector.mozdev.org/rdf/2005/prefix');
    var nsEnum = nsDS.GetSources(rdfType, rdfNamespace, true);
    while (nsEnum.hasMoreElements()) {
        var ns = nsEnum.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
        var prefix = nsDS.GetTarget(ns, rdfPrefix, true).QueryInterface(Components.interfaces.nsIRDFLiteral);

        if (RDFInspectorSettings.HasNamespacePrefix(prefix.Value)) {
            RDFInspectorSettings.RemoveNamespace(prefix.Value);
        }

        RDFInspectorSettings.SetNamespace(prefix.Value, ns.Value, true);
    }
    nsTreeView.update();
}

function editNamespace(aEvent) {
    if (namespaceTree.view.selection.count != 1) return;
    var ns = nsTreeView.contents[namespaceTree.view.selection.currentIndex];
    window.openDialog(
        "chrome://rdfinspector/content/editNamespacePrompt.xul",
        "editNamespacePrompt",
        "chrome",
        namespaceTree.view.selection.currentIndex,
        ns.prefix,
        ns.uri,
        ns.enabled,
        setNamespace
    );
}

function removeNamespace(aEvent) {
    dump('removeNamespace("' + aEvent + '")\n');
    var selection = namespaceTree.view.selection;
    dump('  selection = "' + selection + '"\n');
    var numRanges = selection.getRangeCount();
    dump('  numRanges = "' + numRanges + '"\n');
    for (var rangeIdx = 0; rangeIdx < numRanges; rangeIdx++) {
        dump('  rangeIdx = "' + rangeIdx + '"\n');
        var min = {};
        var max = {};
        selection.getRangeAt(rangeIdx, min, max);
        for (var rowIdx = min.value; rowIdx <= max.value; rowIdx++) {
            dump('  rowIdx = "' + rowIdx + '"\n');
            dump('  nsTreeView.contents[rowIdx].prefix = "' + nsTreeView.contents[rowIdx].prefix + '"\n');
            RDFInspectorSettings.RemoveNamespace(nsTreeView.contents[rowIdx].prefix);
            dump('  Removed\n');
        }
    }
    nsTreeView.update();
}

function newNamespace(aEvent) {
    window.openDialog("chrome://rdfinspector/content/editNamespacePrompt.xul", "editNamespacePrompt", "chrome", null, null, null, true, setNamespace);
}

function setNamespace(aIndex, aPrefix, aURI, aEnabled) {
    if (aIndex) {
        if (RDFInspectorSettings.HasNamespacePrefix(nsTreeView.contents[aIndex].prefix)) {
            RDFInspectorSettings.RemoveNamespace(nsTreeView.contents[aIndex].prefix);
        }
    }

    RDFInspectorSettings.SetNamespace(aPrefix, aURI, aEnabled);
    nsTreeView.update();
}

function nsNamespaceTreeView(aContents) {
    this._contents = aContents;
}
nsNamespaceTreeView.prototype = {
    _contents : null,
    get contents() {
        return this._contents;
    },
    set contents(aContents) {
        this._contents = aContents;
    },
    update : function() {
        var oldLength = 0;
        if (this._contents)
            oldLength = this._contents.length;

        if (this.treebox)
            this.treebox.beginUpdateBatch();
        this._contents = RDFInspectorSettings.GetAllNamespaces();
        if (this.treebox)
            this.treebox.endUpdateBatch();
    },

    get rowCount() {
        return this._contents.length;
    },
    getCellText : function(aRow, aColumn) {
        if (aColumn == "prefixCol") return this._contents[aRow].prefix;
        else if (aColumn == "uriCol") return this._contents[aRow].uri;
        else if (aColumn == "enabledCol") return this._contents[aRow].enabled;
    },
    setTree: function(aTreebox) { this.treebox = aTreebox; },
    isContainer: function(aRow) { return false; },
    isSeparator: function(aRow) { return false; },
    isSorted: function(aRow) { return false; },
    getLevel: function(aRow) { return 0; },
    getImageSrc: function(aRow, aCol) { return null; },
    getRowProperties: function(aRow, aProps) {},
    getCellProperties: function(aRow, aCol, aProps) {},
    getColumnProperties: function(aColid, aCol, aProps) {}
};
